# Copyright 2011-2012 Paul Wouters <paul@nohats.ca>
# Based on http://code.google.com/p/geolocate-cli/ by Francis Markham
#
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License, or 
# (at your option) any later version. This program is distributed in the 
# hope that it will be useful, but WITHOUT ANY WARRANTY; without 
# even the implied warranty of MERCHANTABILITY or FITNESS FOR 
# A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details. You should have received a copy of the GNU General 
# Public License along with this program. If not, see <http://www.gnu.org/licenses/>. 

"""  Obtain geo information via Google's API
This module allows you to obtain geo information about your current location.
It uses NetworkManager to obtain a list of wifi networks, and queries the
Google location service at https://www.google.com/loc/json for geo information.

Example use
-----------

>>> import geome
>>> geome.location()

"""

import dbus, sys, os, urllib2
try:
    import json
except ImportError:
    import simplejson as json 

__version__ = "1.4"

def locate():
	self.location()

def location():
	__NM_DEVICE_TYPE_802_11_WIRELESS = 2
	__url = "https://www.google.com/loc/json"
	__useragent = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.8) Gecko/20110405 Fedora/1x Firefox/3.5"

	__devices = ""
	try:
		__bus = dbus.SystemBus()
		__nm = __bus.get_object('org.freedesktop.NetworkManager', '/org/freedesktop/NetworkManager')
		__devices = __nm.GetDevices()
	except:
		pass

	__aps = []
	for __path in __devices:
		__device = __bus.get_object('org.freedesktop.NetworkManager', __path)
		__device_props = dbus.Interface(__device, dbus_interface='org.freedesktop.DBus.Properties')
		__props = __device_props.GetAll("org.freedesktop.NetworkManager.Device")

		# Different magic seems to be required on different versions of networkmanager
		try:
			__devtype = __props["DeviceType"]
		except KeyError:
			__devtype = device.Get("org.freedesktop.NetworkManager.Device", 'DeviceType')

		if __devtype != __NM_DEVICE_TYPE_802_11_WIRELESS:
	                    continue

		__access_points = __device.GetAccessPoints()

		for __network_path in __access_points:
			if not __network_path:
				# No active network
				continue
			__network = __bus.get_object('org.freedesktop.NetworkManager', __network_path)
			__network_props = dbus.Interface(__network, dbus_interface='org.freedesktop.DBus.Properties')
			__ssid = __network_props.Get("org.freedesktop.NetworkManager.AccessPoint", "Ssid", byte_arrays=True)
			__mac = __network_props.Get("org.freedesktop.NetworkManager.AccessPoint", "HwAddress")
			__sig_str = __network_props.Get("org.freedesktop.NetworkManager.AccessPoint", "Strength")
			__aps.append({"ssid": str(__ssid), "mac": str(__mac), "signal_strength": int(__sig_str)})


	__post_str = "{\"version\":\"1.1.0\", \"request_address\":\"true\", \"wifi_towers\":["
	if not __aps:
		__post_str += '{} '

	for __ap in __aps:
		__ap_strs = []
		__pairs = []
		if 'ssid' in __ap:
			__pairs.append("\"ssid\":\"%s\"" % __ap['ssid'])
		if 'mac' in __ap:
			__pairs.append("\"mac_address\":\"%s\"" % __ap['mac'])
		if 'signal_strength' in __ap:
			__pairs.append("\"signal_strength\":\"%d\"" % __ap['signal_strength'])
		__ap_strs.append('{' + ','.join(__pairs) + '}')
		__post_str += ','.join(__ap_strs)
		__post_str += ","
	__post_str = __post_str[:-1] + "]}"

	__req = urllib2.Request(__url, __post_str, {'User-agent' : __useragent })
	__u = urllib2.urlopen(__req)

	return  json.loads(__u.read())

